package de.hft.stuttgart.citydoctor2.webservice.endpoints;

import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;

import javax.persistence.EntityManager;
import javax.ws.rs.Consumes;
import javax.ws.rs.FormParam;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hft.stuttgart.citydoctor2.webservice.database.UserQueries;
import de.hft.stuttgart.citydoctor2.webservice.model.entities.User;
import de.hft.stuttgart.citydoctor2.webservice.utils.HashUtils;
import de.hft.stuttgart.citydoctor2.webservice.utils.HexUtils;
import de.hft.stuttgart.citydoctor2.webservice.utils.HibernateUtils;

@Path("register")
public class RegisterEndpoint {

	private static Logger logger = LogManager.getLogger(RegisterEndpoint.class);
	
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_FORM_URLENCODED)
	public Response register(@FormParam("username") String username, @FormParam("password") String password) {
		EntityManager manager = null;
		try {
			manager = HibernateUtils.createManager();
			User user = UserQueries.getUserForUsername(username, manager);
			if (user != null) {
				return Response.status(Status.CONFLICT).entity("Username already taken").build();
			}
			String salt = createSalt();
			logger.debug("Salt for new User: {}", salt);
			String passwordHash = HashUtils.createPasswordHash(password, salt);
			user = new User();
			user.setMail(username);
			user.setPasswordHash(passwordHash);
			user.setSalt(salt);
			UserQueries.createUser(user, manager);
			return Response.ok().build();
		} catch (NoSuchAlgorithmException e) {
			logger.fatal(e);
			return Response.status(Status.INTERNAL_SERVER_ERROR).build();
		} finally {
			if (manager != null) {
				manager.close();
			}
		}
	}
	
	private String createSalt() {
		SecureRandom sr = new SecureRandom();
		byte[] bytes = new byte[8];
		sr.nextBytes(bytes);
		return HexUtils.encode(bytes);
	}
	

}
